package dev.shadowsoffire.apotheosis.spawn.modifiers;

import com.mojang.serialization.JsonOps;
import com.google.gson.JsonParseException;
import com.google.gson.JsonObject;
import com.mojang.datafixers.util.Pair;
import org.apache.logging.log4j.Logger;
import com.mojang.serialization.DataResult;
import java.util.function.Consumer;
import net.minecraft.nbt.CompoundTag;
import java.util.Objects;
import dev.shadowsoffire.apotheosis.spawn.SpawnerModule;
import com.mojang.serialization.DynamicOps;
import net.minecraft.nbt.NbtOps;
import net.minecraft.network.FriendlyByteBuf;
import dev.shadowsoffire.apotheosis.spawn.spawner.ApothSpawnerTile;

record StatModifier<T>(SpawnerStat<T> stat, T value, T min, T max) {
    public boolean apply(final ApothSpawnerTile tile) {
        return this.stat.apply(this.value, this.min, this.max, tile);
    }
    
    public void write(final FriendlyByteBuf buf) {
        buf.m_130072_(this.stat.getId(), 32);
        final DataResult encodeStart = this.stat.getModifierCodec().encodeStart((DynamicOps)NbtOps.f_128958_, (Object)this);
        final boolean b = false;
        final Logger log = SpawnerModule.LOG;
        Objects.requireNonNull(log);
        buf.m_130079_((CompoundTag)encodeStart.getOrThrow(b, (Consumer)log::error));
    }
    
    public static StatModifier<?> read(final FriendlyByteBuf buf) {
        final SpawnerStat<?> stat = SpawnerStats.REGISTRY.get(buf.m_130136_(32));
        final DataResult decode = stat.getModifierCodec().decode((DynamicOps)NbtOps.f_128958_, (Object)buf.m_130260_());
        final boolean b = false;
        final Logger log = SpawnerModule.LOG;
        Objects.requireNonNull(log);
        return (StatModifier)((Pair)decode.getOrThrow(b, (Consumer)log::error)).getFirst();
    }
    
    public static StatModifier<?> parse(final JsonObject obj) {
        final SpawnerStat<?> stat = SpawnerStats.REGISTRY.get(obj.get("id").getAsString());
        if (stat == null) {
            throw new JsonParseException("Failed to parse a stat modifier - missing or invalid ID");
        }
        final DataResult decode = stat.getModifierCodec().decode((DynamicOps)JsonOps.INSTANCE, (Object)obj);
        final boolean b = false;
        final Logger log = SpawnerModule.LOG;
        Objects.requireNonNull(log);
        return (StatModifier)((Pair)decode.getOrThrow(b, (Consumer)log::error)).getFirst();
    }
}
